@@ -15,10 +15,11 @@
 #  See the License for the specific language governing permissions and
 # limitations under the License.
 ################################################################################
+from typing import Dict, List
 
 from pyflink.common.execution_mode import ExecutionMode
 from pyflink.common.input_dependency_constraint import InputDependencyConstraint
-from pyflink.common.restart_strategy import RestartStrategies
+from pyflink.common.restart_strategy import RestartStrategies, RestartStrategyConfiguration
 from pyflink.java_gateway import get_gateway
 from pyflink.util.utils import load_java_class
 
@@ -69,7 +70,7 @@ class ExecutionConfig(object):
     def __init__(self, j_execution_config):
         self._j_execution_config = j_execution_config
 
-    def enable_closure_cleaner(self):
+    def enable_closure_cleaner(self) -> 'ExecutionConfig':
         """
         Enables the ClosureCleaner. This analyzes user code functions and sets fields to null
         that are not used. This will in most cases make closures or anonymous inner classes
@@ -81,7 +82,7 @@ class ExecutionConfig(object):
         self._j_execution_config = self._j_execution_config.enableClosureCleaner()
         return self
 
-    def disable_closure_cleaner(self):
+    def disable_closure_cleaner(self) -> 'ExecutionConfig':
         """
         Disables the ClosureCleaner.
 
@@ -92,7 +93,7 @@ class ExecutionConfig(object):
         self._j_execution_config = self._j_execution_config.disableClosureCleaner()
         return self
 
-    def is_closure_cleaner_enabled(self):
+    def is_closure_cleaner_enabled(self) -> bool:
         """
         Returns whether the ClosureCleaner is enabled.
 
@@ -102,7 +103,7 @@ class ExecutionConfig(object):
         """
         return self._j_execution_config.isClosureCleanerEnabled()
 
-    def set_auto_watermark_interval(self, interval):
+    def set_auto_watermark_interval(self, interval: int) -> 'ExecutionConfig':
         """
         Sets the interval of the automatic watermark emission. Watermarks are used throughout
         the streaming system to keep track of the progress of time. They are used, for example,
@@ -114,7 +115,7 @@ class ExecutionConfig(object):
         self._j_execution_config = self._j_execution_config.setAutoWatermarkInterval(interval)
         return self
 
-    def get_auto_watermark_interval(self):
+    def get_auto_watermark_interval(self) -> int:
         """
         Returns the interval of the automatic watermark emission.
 
@@ -124,7 +125,7 @@ class ExecutionConfig(object):
         """
         return self._j_execution_config.getAutoWatermarkInterval()
 
-    def set_latency_tracking_interval(self, interval):
+    def set_latency_tracking_interval(self, interval: int) -> 'ExecutionConfig':
         """
         Interval for sending latency tracking marks from the sources to the sinks.
 
@@ -137,7 +138,7 @@ class ExecutionConfig(object):
         self._j_execution_config = self._j_execution_config.setLatencyTrackingInterval(interval)
         return self
 
-    def get_latency_tracking_interval(self):
+    def get_latency_tracking_interval(self) -> int:
         """
         Returns the latency tracking interval.
 
@@ -145,7 +146,7 @@ class ExecutionConfig(object):
         """
         return self._j_execution_config.getLatencyTrackingInterval()
 
-    def get_parallelism(self):
+    def get_parallelism(self) -> int:
         """
         Gets the parallelism with which operation are executed by default. Operations can
         individually override this value to use a specific parallelism.
@@ -160,7 +161,7 @@ class ExecutionConfig(object):
         """
         return self._j_execution_config.getParallelism()
 
-    def set_parallelism(self, parallelism):
+    def set_parallelism(self, parallelism: int) -> 'ExecutionConfig':
         """
         Sets the parallelism for operations executed through this environment.
         Setting a parallelism of x here will cause all operators (such as join, map, reduce) to run
@@ -177,7 +178,7 @@ class ExecutionConfig(object):
         self._j_execution_config = self._j_execution_config.setParallelism(parallelism)
         return self
 
-    def get_max_parallelism(self):
+    def get_max_parallelism(self) -> int:
         """
         Gets the maximum degree of parallelism defined for the program.
 
@@ -188,7 +189,7 @@ class ExecutionConfig(object):
         """
         return self._j_execution_config.getMaxParallelism()
 
-    def set_max_parallelism(self, max_parallelism):
+    def set_max_parallelism(self, max_parallelism: int) -> 'ExecutionConfig':
         """
         Sets the maximum degree of parallelism defined for the program.
 
@@ -198,8 +199,9 @@ class ExecutionConfig(object):
         :param max_parallelism: Maximum degree of parallelism to be used for the program.
         """
         self._j_execution_config.setMaxParallelism(max_parallelism)
+        return self
 
-    def get_task_cancellation_interval(self):
+    def get_task_cancellation_interval(self) -> int:
         """
         Gets the interval (in milliseconds) between consecutive attempts to cancel a running task.
 
@@ -207,7 +209,7 @@ class ExecutionConfig(object):
         """
         return self._j_execution_config.getTaskCancellationInterval()
 
-    def set_task_cancellation_interval(self, interval):
+    def set_task_cancellation_interval(self, interval: int) -> 'ExecutionConfig':
         """
         Sets the configuration parameter specifying the interval (in milliseconds)
         between consecutive attempts to cancel a running task.
@@ -218,7 +220,7 @@ class ExecutionConfig(object):
         self._j_execution_config = self._j_execution_config.setTaskCancellationInterval(interval)
         return self
 
-    def get_task_cancellation_timeout(self):
+    def get_task_cancellation_timeout(self) -> int:
         """
         Returns the timeout (in milliseconds) after which an ongoing task
         cancellation leads to a fatal TaskManager error.
@@ -230,7 +232,7 @@ class ExecutionConfig(object):
         """
         return self._j_execution_config.getTaskCancellationTimeout()
 
-    def set_task_cancellation_timeout(self, timeout):
+    def set_task_cancellation_timeout(self, timeout: int) -> 'ExecutionConfig':
         """
         Sets the timeout (in milliseconds) after which an ongoing task cancellation
         is considered failed, leading to a fatal TaskManager error.
@@ -246,7 +248,9 @@ class ExecutionConfig(object):
         self._j_execution_config = self._j_execution_config.setTaskCancellationTimeout(timeout)
         return self
 
-    def set_restart_strategy(self, restart_strategy_configuration):
+    def set_restart_strategy(
+            self,
+            restart_strategy_configuration: RestartStrategyConfiguration) -> 'ExecutionConfig':
         """
         Sets the restart strategy to be used for recovery.
         ::
@@ -260,8 +264,9 @@ class ExecutionConfig(object):
         """
         self._j_execution_config.setRestartStrategy(
             restart_strategy_configuration._j_restart_strategy_configuration)
+        return self
 
-    def get_restart_strategy(self):
+    def get_restart_strategy(self) -> RestartStrategyConfiguration:
         """
         Returns the restart strategy which has been set for the current job.
 
@@ -272,7 +277,7 @@ class ExecutionConfig(object):
         return RestartStrategies._from_j_restart_strategy(
             self._j_execution_config.getRestartStrategy())
 
-    def set_execution_mode(self, execution_mode):
+    def set_execution_mode(self, execution_mode: ExecutionMode) -> 'ExecutionConfig':
         """
         Sets the execution mode to execute the program. The execution mode defines whether
         data exchanges are performed in a batch or on a pipelined manner.
@@ -290,10 +295,10 @@ class ExecutionConfig(object):
                                :data:`ExecutionMode.BATCH` or
                                :data:`ExecutionMode.BATCH_FORCED`.
         """
-        self._j_execution_config.setExecutionMode(
-            ExecutionMode._to_j_execution_mode(execution_mode))
+        self._j_execution_config.setExecutionMode(execution_mode._to_j_execution_mode())
+        return self
 
-    def get_execution_mode(self):
+    def get_execution_mode(self) -> 'ExecutionMode':
         """
         Gets the execution mode used to execute the program. The execution mode defines whether
         data exchanges are performed in a batch or on a pipelined manner.
@@ -307,7 +312,8 @@ class ExecutionConfig(object):
         j_execution_mode = self._j_execution_config.getExecutionMode()
         return ExecutionMode._from_j_execution_mode(j_execution_mode)
 
-    def set_default_input_dependency_constraint(self, input_dependency_constraint):
+    def set_default_input_dependency_constraint(
+            self, input_dependency_constraint: InputDependencyConstraint) -> 'ExecutionConfig':
         """
         Sets the default input dependency constraint for vertex scheduling. It indicates when a
         task should be scheduled considering its inputs status.
@@ -324,10 +330,10 @@ class ExecutionConfig(object):
                                             :data:`InputDependencyConstraint.ALL`.
         """
         self._j_execution_config.setDefaultInputDependencyConstraint(
-            InputDependencyConstraint._to_j_input_dependency_constraint(
-                input_dependency_constraint))
+            input_dependency_constraint._to_j_input_dependency_constraint())
+        return self
 
-    def get_default_input_dependency_constraint(self):
+    def get_default_input_dependency_constraint(self) -> 'InputDependencyConstraint':
         """
         Gets the default input dependency constraint for vertex scheduling. It indicates when a
         task should be scheduled considering its inputs status.
@@ -344,28 +350,30 @@ class ExecutionConfig(object):
         return InputDependencyConstraint._from_j_input_dependency_constraint(
             j_input_dependency_constraint)
 
-    def enable_force_kryo(self):
+    def enable_force_kryo(self) -> 'ExecutionConfig':
         """
         Force TypeExtractor to use Kryo serializer for POJOS even though we could analyze as POJO.
         In some cases this might be preferable. For example, when using interfaces
         with subclasses that cannot be analyzed as POJO.
         """
         self._j_execution_config.enableForceKryo()
+        return self
 
-    def disable_force_kryo(self):
+    def disable_force_kryo(self) -> 'ExecutionConfig':
         """
         Disable use of Kryo serializer for all POJOs.
         """
         self._j_execution_config.disableForceKryo()
+        return self
 
-    def is_force_kryo_enabled(self):
+    def is_force_kryo_enabled(self) -> bool:
         """
         :return: Boolean value that represent whether the usage of Kryo serializer for all POJOs
                  is enabled.
         """
         return self._j_execution_config.isForceKryoEnabled()
 
-    def enable_generic_types(self):
+    def enable_generic_types(self) -> 'ExecutionConfig':
         """
         Enables the use generic types which are serialized via Kryo.
 
@@ -374,8 +382,9 @@ class ExecutionConfig(object):
         .. seealso:: :func:`disable_generic_types`
         """
         self._j_execution_config.enableGenericTypes()
+        return self
 
-    def disable_generic_types(self):
+    def disable_generic_types(self) -> 'ExecutionConfig':
         """
         Disables the use of generic types (types that would be serialized via Kryo). If this option
         is used, Flink will throw an ``UnsupportedOperationException`` whenever it encounters
@@ -394,8 +403,9 @@ class ExecutionConfig(object):
         .. seealso:: :func:`enable_generic_types`
         """
         self._j_execution_config.disableGenericTypes()
+        return self
 
-    def has_generic_types_disabled(self):
+    def has_generic_types_disabled(self) -> bool:
         """
         Checks whether generic types are supported. Generic types are types that go through Kryo
         during serialization.
@@ -410,15 +420,16 @@ class ExecutionConfig(object):
         """
         return self._j_execution_config.hasGenericTypesDisabled()
 
-    def enable_auto_generated_uids(self):
+    def enable_auto_generated_uids(self) -> 'ExecutionConfig':
         """
         Enables the Flink runtime to auto-generate UID's for operators.
 
         .. seealso:: :func:`disable_auto_generated_uids`
         """
         self._j_execution_config.enableAutoGeneratedUIDs()
+        return self
 
-    def disable_auto_generated_uids(self):
+    def disable_auto_generated_uids(self) -> 'ExecutionConfig':
         """
         Disables auto-generated UIDs. Forces users to manually specify UIDs
         on DataStream applications.
@@ -430,8 +441,9 @@ class ExecutionConfig(object):
         without discarding state.
         """
         self._j_execution_config.disableAutoGeneratedUIDs()
+        return self
 
-    def has_auto_generated_uids_enabled(self):
+    def has_auto_generated_uids_enabled(self) -> bool:
         """
         Checks whether auto generated UIDs are supported.
 
@@ -445,21 +457,23 @@ class ExecutionConfig(object):
         """
         return self._j_execution_config.hasAutoGeneratedUIDsEnabled()
 
-    def enable_force_avro(self):
+    def enable_force_avro(self) -> 'ExecutionConfig':
         """
         Forces Flink to use the Apache Avro serializer for POJOs.
 
         **Important:** Make sure to include the *flink-avro* module.
         """
         self._j_execution_config.enableForceAvro()
+        return self
 
-    def disable_force_avro(self):
+    def disable_force_avro(self) -> 'ExecutionConfig':
         """
         Disables the Apache Avro serializer as the forced serializer for POJOs.
         """
         self._j_execution_config.disableForceAvro()
+        return self
 
-    def is_force_avro_enabled(self):
+    def is_force_avro_enabled(self) -> bool:
         """
         Returns whether the Apache Avro is the default serializer for POJOs.
 
@@ -468,7 +482,7 @@ class ExecutionConfig(object):
         """
         return self._j_execution_config.isForceAvroEnabled()
 
-    def enable_object_reuse(self):
+    def enable_object_reuse(self) -> 'ExecutionConfig':
         """
         Enables reusing objects that Flink internally uses for deserialization and passing
         data to user-code functions. Keep in mind that this can lead to bugs when the
@@ -479,7 +493,7 @@ class ExecutionConfig(object):
         self._j_execution_config = self._j_execution_config.enableObjectReuse()
         return self
 
-    def disable_object_reuse(self):
+    def disable_object_reuse(self) -> 'ExecutionConfig':
         """
         Disables reusing objects that Flink internally uses for deserialization and passing
         data to user-code functions.
@@ -491,7 +505,7 @@ class ExecutionConfig(object):
         self._j_execution_config = self._j_execution_config.disableObjectReuse()
         return self
 
-    def is_object_reuse_enabled(self):
+    def is_object_reuse_enabled(self) -> bool:
         """
         Returns whether object reuse has been enabled or disabled.
 
@@ -501,33 +515,7 @@ class ExecutionConfig(object):
         """
         return self._j_execution_config.isObjectReuseEnabled()
 
-    def enable_sysout_logging(self):
-        """
-        Enables the printing of progress update messages to stdout.
-
-        :return: This object.
-        """
-        self._j_execution_config = self._j_execution_config.enableSysoutLogging()
-        return self
-
-    def disable_sysout_logging(self):
-        """
-        Disables the printing of progress update messages to stdout.
-
-        :return: This object.
-        """
-        self._j_execution_config = self._j_execution_config.disableSysoutLogging()
-        return self
-
-    def is_sysout_logging_enabled(self):
-        """
-        Gets whether progress update messages should be printed to stdout.
-
-        :return: True, if progress update messages should be printed, false otherwise.
-        """
-        return self._j_execution_config.isSysoutLoggingEnabled()
-
-    def get_global_job_parameters(self):
+    def get_global_job_parameters(self) -> Dict[str, str]:
         """
         Gets current configuration dict.
 
@@ -535,7 +523,7 @@ class ExecutionConfig(object):
         """
         return dict(self._j_execution_config.getGlobalJobParameters().toMap())
 
-    def set_global_job_parameters(self, global_job_parameters_dict):
+    def set_global_job_parameters(self, global_job_parameters_dict: Dict) -> 'ExecutionConfig':
         """
         Register a custom, serializable user configuration dict.
 
@@ -556,8 +544,11 @@ class ExecutionConfig(object):
                 value = global_job_parameters_dict[key]
             j_global_job_parameters.setString(key, value)
         self._j_execution_config.setGlobalJobParameters(j_global_job_parameters)
+        return self
 
-    def add_default_kryo_serializer(self, type_class_name, serializer_class_name):
+    def add_default_kryo_serializer(self,
+                                    type_class_name: str,
+                                    serializer_class_name: str) -> 'ExecutionConfig':
         """
         Adds a new Kryo default serializer to the Runtime.
 
@@ -574,8 +565,11 @@ class ExecutionConfig(object):
         type_clz = load_java_class(type_class_name)
         j_serializer_clz = load_java_class(serializer_class_name)
         self._j_execution_config.addDefaultKryoSerializer(type_clz, j_serializer_clz)
+        return self
 
-    def register_type_with_kryo_serializer(self, type_class_name, serializer_class_name):
+    def register_type_with_kryo_serializer(self,
+                                           type_class_name: str,
+                                           serializer_class_name: str) -> 'ExecutionConfig':
         """
         Registers the given Serializer via its class as a serializer for the given type at the
         KryoSerializer.
@@ -593,8 +587,9 @@ class ExecutionConfig(object):
         type_clz = load_java_class(type_class_name)
         j_serializer_clz = load_java_class(serializer_class_name)
         self._j_execution_config.registerTypeWithKryoSerializer(type_clz, j_serializer_clz)
+        return self
 
-    def register_pojo_type(self, type_class_name):
+    def register_pojo_type(self, type_class_name: str) -> 'ExecutionConfig':
         """
         Registers the given type with the serialization stack. If the type is eventually
         serialized as a POJO, then the type is registered with the POJO serializer. If the
@@ -610,8 +605,9 @@ class ExecutionConfig(object):
         """
         type_clz = load_java_class(type_class_name)
         self._j_execution_config.registerPojoType(type_clz)
+        return self
 
-    def register_kryo_type(self, type_class_name):
+    def register_kryo_type(self, type_class_name: str) -> 'ExecutionConfig':
         """
         Registers the given type with the serialization stack. If the type is eventually
         serialized as a POJO, then the type is registered with the POJO serializer. If the
@@ -627,8 +623,9 @@ class ExecutionConfig(object):
         """
         type_clz = load_java_class(type_class_name)
         self._j_execution_config.registerKryoType(type_clz)
+        return self
 
-    def get_registered_types_with_kryo_serializer_classes(self):
+    def get_registered_types_with_kryo_serializer_classes(self) -> Dict[str, str]:
         """
         Returns the registered types with their Kryo Serializer classes.
 
@@ -642,7 +639,7 @@ class ExecutionConfig(object):
             registered_serializers[key.getName()] = j_clz_map[key].getName()
         return registered_serializers
 
-    def get_default_kryo_serializer_classes(self):
+    def get_default_kryo_serializer_classes(self) -> Dict[str, str]:
         """
         Returns the registered default Kryo Serializer classes.
 
@@ -656,7 +653,7 @@ class ExecutionConfig(object):
             default_kryo_serializers[key.getName()] = j_clz_map[key].getName()
         return default_kryo_serializers
 
-    def get_registered_kryo_types(self):
+    def get_registered_kryo_types(self) -> List[str]:
         """
         Returns the registered Kryo types.
 
@@ -665,7 +662,7 @@ class ExecutionConfig(object):
         j_clz_set = self._j_execution_config.getRegisteredKryoTypes()
         return [value.getName() for value in j_clz_set]
 
-    def get_registered_pojo_types(self):
+    def get_registered_pojo_types(self) -> List[str]:
         """
         Returns the registered POJO types.
 
@@ -674,7 +671,7 @@ class ExecutionConfig(object):
         j_clz_set = self._j_execution_config.getRegisteredPojoTypes()
         return [value.getName() for value in j_clz_set]
 
-    def is_auto_type_registration_disabled(self):
+    def is_auto_type_registration_disabled(self) -> bool:
         """
         Returns whether Flink is automatically registering all types in the user programs with
         Kryo.
@@ -683,14 +680,15 @@ class ExecutionConfig(object):
         """
         return self._j_execution_config.isAutoTypeRegistrationDisabled()
 
-    def disable_auto_type_registration(self):
+    def disable_auto_type_registration(self) -> 'ExecutionConfig':
         """
         Control whether Flink is automatically registering all types in the user programs with
         Kryo.
         """
         self._j_execution_config.disableAutoTypeRegistration()
+        return self
 
-    def is_use_snapshot_compression(self):
+    def is_use_snapshot_compression(self) -> bool:
         """
         Returns whether he compression (snappy) for keyed state in full checkpoints and savepoints
         is enabled.
@@ -699,7 +697,7 @@ class ExecutionConfig(object):
         """
         return self._j_execution_config.isUseSnapshotCompression()
 
-    def set_use_snapshot_compression(self, use_snapshot_compression):
+    def set_use_snapshot_compression(self, use_snapshot_compression: bool) -> 'ExecutionConfig':
         """
         Control whether the compression (snappy) for keyed state in full checkpoints and savepoints
         is enabled.
@@ -707,6 +705,7 @@ class ExecutionConfig(object):
         :param use_snapshot_compression: ``True`` means enabled and ``False`` means disabled.
         """
         self._j_execution_config.setUseSnapshotCompression(use_snapshot_compression)
+        return self
 
     def __eq__(self, other):
         return isinstance(other, self.__class__) and \
